﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;

namespace Tasks.Services
{
    public class TaskManagerService : ITaskManagerService
    {
        public int AddTask(string taskDescription, string assignedTo)
        {
            int taskNum = s_nextTaskNumber++;
            TaskInfo ti = new TaskInfo(taskNum, taskDescription, assignedTo);
            Console.WriteLine("Adding new task:\n{0}", ti.ToString());
            s_activeTasks.Add(taskNum, ti);
            return taskNum;
        }

        public List<int> GetTasksByAssignedName(string assignedTo)
        {
            var tasks = from ti in s_activeTasks
                        where ti.Value.AssignedTo == assignedTo
                        select ti.Key;

            Console.WriteLine("Get the following tasks:");
            foreach (var t in tasks)
                Console.WriteLine(t);

            return tasks.ToList();
        }

        public string GetTaskDescription(int taskNumber)
        {
            if (!s_activeTasks.ContainsKey(taskNumber))
            {
                string msg = string.Format("No task with number {0}", taskNumber);
                Console.WriteLine(msg);

                // throw fault
                throw new Exception(msg);
            }

            string descr = s_activeTasks[taskNumber].Description;

            Console.WriteLine("Description for task number {0} is {1}", taskNumber, descr);

            return descr;
        }

        public bool IsTaskCompleted(int taskNumber)
        {
            if (!s_activeTasks.ContainsKey(taskNumber))
            {
                string msg = string.Format("No task with number {0}", taskNumber);
                Console.WriteLine(msg);

                // throw fault
                throw new Exception(msg);
            }

            bool res = s_activeTasks[taskNumber].IsCompleted;

            Console.WriteLine("Task number {0} is {1}", taskNumber, res ? "Completed" : "Not Completed");

            return res;
        }

        public void MarkTaskCompleted(int taskNumber)
        {
            if (!s_activeTasks.ContainsKey(taskNumber))
            {
                string msg = string.Format("No task with number {0}", taskNumber);
                Console.WriteLine(msg);

                // throw fault
                throw new Exception("No task with that number");
            }

            s_activeTasks[taskNumber].IsCompleted = true;

            Console.WriteLine("Marked task number {0} as completed", taskNumber);
        }

        public void DeleteTask(int taskNumber)
        {
            if (!s_activeTasks.ContainsKey(taskNumber))
            {
                string msg = string.Format("No task with number {0}", taskNumber);
                Console.WriteLine(msg);

                // throw fault
                throw new Exception("No task with that number");
            }

            s_activeTasks.Remove(taskNumber);

            Console.WriteLine("Task number {0} as deleted", taskNumber);
        }

        private static Dictionary<int, TaskInfo> s_activeTasks = new Dictionary<int, TaskInfo>();
        private static int s_nextTaskNumber = 1;
    }

    class TaskInfo
    {
        public TaskInfo(int taskNumber, string description, string assignedTo)
        {
            _taskNumber = taskNumber;
            _description = description;
            _assignedTo = assignedTo;
        }

        public int TaskNumber
        {
            get
            {
                return _taskNumber;
            }
        }

        public string Description
        {
            get
            {
                return _description;
            }
        }

        public string AssignedTo
        {
            get
            {
                return _assignedTo;
            }
        }

        public bool IsCompleted
        {
            get
            {
                return _isCompleted;
            }

            set
            {
                _isCompleted = value;
            }
        }

        public override string ToString()
        {
            return string.Format(
                    "Task: {0}\tAssigned to: {1}\tCompleted: {2}\nDescription:\n{3}",
                    _taskNumber, _assignedTo,
                    _isCompleted, _description);
        }

        private int _taskNumber;
        private string _description;
        private string _assignedTo;
        private bool _isCompleted = false;
    }
}
