﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Channels;
using System.ServiceModel;

namespace DemoService
{
    public class DemoContextProvider : IInstanceContextProvider
    {
        Dictionary<String, DemoContextInfo> contextMap = new Dictionary<string,DemoContextInfo>();
        string headerName = "RequesterInstanceId";
        string headerNamespace = "http://tempuri.org/Shareable";

        #region IInstanceContextProvider Members

        public InstanceContext GetExistingInstanceContext(Message message, IContextChannel channel)
        {
            System.Diagnostics.Trace.WriteLine("In GetExistingInstanceContext");
            DemoContextInfo info;

            bool hasSession = (channel.SessionId != null);
            if (hasSession)
            {
                info = channel.Extensions.Find<DemoContextInfo>();
            }

            int headerIndex = message.Headers.FindHeader(headerName, headerNamespace);

            string instanceId = null;
            if (headerIndex != -1)
                instanceId = message.Headers.GetHeader<string>(headerIndex);

            bool isNew = false;

            lock (this.contextMap)
            {
                if ((instanceId == null) || !this.contextMap.TryGetValue(instanceId, out info))
                {
                    info = new DemoContextInfo(instanceId);
                    isNew = true;
                    this.contextMap.Add(instanceId, info);
                }
            }

            if (hasSession)
            {
                channel.Extensions.Add(info);
            }

            if (isNew)
            {
                return null;
            }
            else
            {
                InstanceContext instanceContext = info.Instance;
                if (hasSession)
                {
                    instanceContext.IncomingChannels.Add(channel);
                }
                return instanceContext;
            }
        }

        public void InitializeInstanceContext(System.ServiceModel.InstanceContext instanceContext, System.ServiceModel.Channels.Message message, System.ServiceModel.IContextChannel channel)
        {
            System.Diagnostics.Trace.WriteLine("In InitializeInstanceContext");
            DemoContextInfo info = null;
            bool hasSession = (channel.SessionId != null);

            if (hasSession)
            {
                instanceContext.IncomingChannels.Add(channel);

                info = channel.Extensions.Find<DemoContextInfo>();
            }
            else
            {
                int headerIndex = message.Headers.FindHeader(headerName, headerNamespace);
                if (headerIndex != -1)
                {
                    string instanceId = message.Headers.GetHeader<string>(headerIndex);
                    if (instanceId != null)
                        this.contextMap.TryGetValue(instanceId, out info);
                }
            }

            if (info != null)
            {
                //instanceContext.Extensions.Add(info);
                info.Instance = instanceContext;
            }

            instanceContext.Closing += delegate(object sender, EventArgs e)
            {
                lock (this.contextMap)
                {
                    this.contextMap.Remove(info.InstanceId);
                }
            };
        }

        public bool IsIdle(System.ServiceModel.InstanceContext instanceContext)
        {
            return false;
        }

        public void NotifyIdle(InstanceContextIdleCallback callback, System.ServiceModel.InstanceContext instanceContext)
        {
        }

        #endregion

    }
}
