﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Threading;

namespace ThreadPoolSynchronization
{
    internal class WorkerThread
    {
        ThreadPoolSynchronizer context;
        public Thread threadObj;
        bool endLoop;

        public int ManagedThreadId
        {
            get
            {
                return threadObj.ManagedThreadId;
            }
        }

        internal WorkerThread(int threadNumber, ThreadPoolSynchronizer context)
        {
            this.context = context;

            endLoop = false;
            threadObj = null;

            threadObj = new Thread(Run);
            threadObj.IsBackground = true;
            threadObj.Name = "Tread-" + threadNumber.ToString();
            threadObj.Start();
        }

        bool EndLoop
        {
            set
            {
                lock (this)
                {
                    endLoop = value;
                }
            }
            get
            {
                lock (this)
                {
                    return endLoop;
                }
            }
        }

        void Start()
        {
            Debug.Assert(threadObj != null);
            Debug.Assert(threadObj.IsAlive == false);
            threadObj.Start();
        }

        void Run()
        {
            Debug.Assert(SynchronizationContext.Current == null);
            SynchronizationContext.SetSynchronizationContext(context);

            while (EndLoop == false)
            {
                WorkItem workItem = context.GetNext();
                if (workItem != null)
                {
                    workItem.CallBack();
                }
            }
        }

        public void Abort()
        {
            Debug.Assert(threadObj != null);
            if (threadObj.IsAlive == false)
            {
                return;
            }
            EndLoop = true;

            threadObj.Join();
        }
    }

}
