﻿Imports System.ServiceModel
Imports System.Linq

Public Class TaskManagerService
    Implements ITaskManagerService
    Public Function AddTask(ByVal taskDescription As String, _
                ByVal assignedTo As String) _
                As Integer Implements ITaskManagerService.AddTask
        Dim taskNum As Integer = s_nextTaskNumber
        s_nextTaskNumber = s_nextTaskNumber + 1
        Dim ti As New TaskInfo(taskNum, taskDescription, assignedTo)
        Console.WriteLine("Adding new task:\n{0}", ti.ToString())
        s_activeTasks.Add(taskNum, ti)
        Return taskNum
    End Function

    Public Function GetTasksByAssignedName( _
            ByVal assignedTo As String) As List(Of Integer) _
            Implements ITaskManagerService.GetTasksByAssignedName
        Dim tasks = From ti In s_activeTasks _
                    Where ti.Value.AssignedTo = assignedTo _
                    Select ti.Key

        Console.WriteLine("Get the following tasks:")

        For Each t In tasks
            Console.WriteLine(t)
        Next

        Return tasks.ToList()

    End Function

    Public Function GetTaskDescription(ByVal taskNumber As Integer) _
        As String Implements ITaskManagerService.GetTaskDescription
        If Not s_activeTasks.ContainsKey(taskNumber) Then
            Dim msg As String = String.Format( _
                "No task with number {0}", taskNumber)
            Console.WriteLine(msg)
            Throw New Exception(msg)
        End If

        Dim descr As String = s_activeTasks.Item(taskNumber).Description
        Console.WriteLine("Description for task number {0} is {1}", _
                          taskNumber, descr)

        Return descr

    End Function

    ' Etc...
    Public Function IsTaskCompleted(ByVal taskNumber As Integer) _
        As Boolean Implements ITaskManagerService.IsTaskCompleted

        If Not s_activeTasks.ContainsKey(taskNumber) Then
            Dim msg As String = String.Format("No task with number {0}", taskNumber)
            Console.WriteLine(msg)
            Throw New Exception(msg)
        End If

        Dim res As Boolean = s_activeTasks.Item(taskNumber).IsCompleted

        Dim taskStatuStr As String
        If res Then
            taskStatuStr = "Completed"
        Else
            taskStatuStr = "Not Completed"
        End If

        Console.WriteLine("Task number {0} is {1}", taskNumber, taskStatuStr)

        Return res
    End Function

    Public Sub MarkTaskCompleted(ByVal taskNumber As Integer) _
        Implements ITaskManagerService.MarkTaskCompleted

        If Not s_activeTasks.ContainsKey(taskNumber) Then
            Dim msg As String = String.Format("No task with number {0}", taskNumber)
            Console.WriteLine(msg)
            Throw New Exception(msg)
        End If

        s_activeTasks.Item(taskNumber).IsCompleted = True
        Console.WriteLine("Marked task number {0} as completed", taskNumber)

    End Sub

    Public Sub DeleteTask(ByVal taskNumber As Integer) _
        Implements ITaskManagerService.DeleteTask

        If Not s_activeTasks.ContainsKey(taskNumber) Then
            Dim msg As String = String.Format("No task with number {0}", taskNumber)
            Console.WriteLine(msg)
            Throw New Exception(msg)
        End If

        s_activeTasks.Remove(taskNumber)
        Console.WriteLine("Task number {0} as deleted", taskNumber)

    End Sub

    Private Shared s_activeTasks As Dictionary(Of Integer, TaskInfo) = New Dictionary(Of Integer, TaskInfo)
    Private Shared s_nextTaskNumber As Integer = 1
End Class



Class TaskInfo
    Public Sub New(ByVal taskNumber As Integer, _
            ByVal description As String, ByVal assignedTo As String)
        _taskNumber = taskNumber
        _description = description
        _assignedTo = assignedTo
    End Sub
    Public Property TaskNumber() As Integer
        Get
            Return _taskNumber
        End Get

        Set(ByVal value As Integer)
            _taskNumber = value
        End Set
    End Property

    Public Property Description() As String
        Get
            Return _description
        End Get

        Set(ByVal value As String)
            _description = value
        End Set
    End Property

    Public Property AssignedTo() As String
        Get
            Return _assignedTo
        End Get

        Set(ByVal value As String)
            _assignedTo = value
        End Set
    End Property

    Public Property IsCompleted() As Boolean
        Get
            Return _isCompleted
        End Get

        Set(ByVal value As Boolean)
            _isCompleted = value
        End Set
    End Property

    Public Overrides Function ToString() As String
        Return String.Format( _
        "Task: {0}\tAssigned to: {1}\tCompleted: {2}\nDescription:\n{3}", _
                _taskNumber, _assignedTo, _
                _isCompleted, _description)
    End Function

    Private _description As String
    Private _assignedTo As String
    Private _taskNumber As Integer
    Private _isCompleted As Boolean = False
End Class