﻿using System;
using System.ServiceModel;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using TaskClient.TaskService.Proxy;

namespace TaskClient
{
	public partial class TaskForm : Form
	{
		private TaskManagerServiceClient _taskMgrProxy = new TaskManagerServiceClient();
		private string _name;

		public TaskForm( string name)
		{
			InitializeComponent();

			_name = name;
		}

		private void _btnSubmitTask_Click(object sender, EventArgs e)
		{
            Task newTask = new Task();
            newTask.AssignedTo = _txtAssignToName.Text;
            newTask.Description = _rtbNewTaskDescription.Text;
            newTask.CreatedBy = _name;
            newTask.DateCreated = DateTime.Now;
            newTask.DateLastModified = DateTime.Now;
            newTask.DueDate = _dateTimePicker.Value;

			TaskAcknowledgement taskAck = _taskMgrProxy.AddTask(newTask);

            newTask.TaskState = taskAck.CurrentState;
			_statusLabel.Text = string.Format("Task number {0} was submitted. {1}", 
                                taskAck.TaskNumber,
                                taskAck.Comments != null ? taskAck.Comments : string.Empty);

			_rtbNewTaskDescription.Text = string.Empty;
			_txtAssignToName.Text = string.Empty;

			_rtbNewTaskDescription.Focus();

			GetTasksFromService();
		}

		private void _taskList_SelectedIndexChanged(object sender, EventArgs e)
		{
            Task selTask = _taskList.SelectedItem as Task;

            _checkBoxCompleted.Checked = (selTask.TaskState == TaskStates.CompletedByDueDate || selTask.TaskState == TaskStates.CompletedPastDueDate);
            _rtbTaskDescription.Text = selTask.Description;
		}

		private void _checkBoxCompleted_CheckedChanged(object sender, EventArgs e)
		{
			if (_checkBoxCompleted.Checked)
			{
                Task selTask = _taskList.SelectedItem as Task;

                if (selTask.TaskState == TaskStates.CompletedByDueDate ||
                        selTask.TaskState == TaskStates.CompletedPastDueDate)
                {
                    return;
                }

				try
				{
					_taskMgrProxy.MarkTaskCompleted(selTask.TaskNumber);

                    // Mark it locally as well:
                    if (DateTime.Now > selTask.DueDate)
                        selTask.TaskState = TaskStates.CompletedPastDueDate;
                    else
                        selTask.TaskState = TaskStates.CompletedByDueDate;
                }

				catch (FaultException<FaultInfo> fi)
				{
					MessageBox.Show(
						"There was a problem mark a task completed with the service.\n" +
							fi.Detail.Reason);
				}
			}
		}

		private void TaskForm_Load(object sender, EventArgs e)
		{
			this.Text = "Task Manager: " + _name;

			GetTasksFromService();
		}

		private void GetTasksFromService( )
		{
			// First clear the list...
			_taskList.Items.Clear();

            _taskList.DisplayMember = "TaskNumber";

			// And then load it:
			Task[] tasks = _taskMgrProxy.GetTasksByAssignedName(_name);
			foreach( Task t in tasks)
				_taskList.Items.Add(t);
		}

		private void _btnDeleteTask_Click(object sender, EventArgs e)
		{
			Task selTask = _taskList.SelectedItem as Task;

			try
			{
				_taskMgrProxy.DeleteTask(selTask.TaskNumber);

				GetTasksFromService();

				if (_taskList.Items.Count > 0)
				{
					_taskList.SelectedIndex = 0;
				}
				else
				{
					_rtbTaskDescription.Text = string.Empty;
					_checkBoxCompleted.Checked = false;
				}
			}

			catch (FaultException<FaultInfo> fi)
			{
				MessageBox.Show(
					"There was a problem deleting a task with the service.\n" +
						fi.Detail.Reason + "\nPlease try again later.");
			}
		}
	}
}
