﻿Imports System.ServiceModel
Imports System.Linq
Imports Tasks.Entities

Public Class TaskManagerService
    Implements ITaskManagerService
    Public Function AddTask(ByVal task As Task) _
            As TaskAcknowledgement _
                Implements ITaskManagerService.AddTask
        Dim taskNum As Integer = s_nextTaskNumber
        s_nextTaskNumber = s_nextTaskNumber + 1

        Dim ack As New TaskAcknowledgement()
        ack.TaskNumber = taskNum
        task.TaskNumber = taskNum

        If task.DueDate > DateTime.Now Then
            task.TaskState = TaskStates.Active
        Else
            task.TaskState = TaskStates.Overdue
        End If

        ack.CurrentState = task.TaskState
        If ack.CurrentState = TaskStates.Overdue Then
            ack.Comments = "Warning: task is already overdue!"
        End If

        Console.WriteLine( _
                "Adding new task:\n{0}", task.ToString())
        s_activeTasks.Add(taskNum, task)
        Return ack
    End Function

    Public Function GetTasksByAssignedName( _
            ByVal assignedTo As String) As List(Of Task) _
            Implements ITaskManagerService.GetTasksByAssignedName
        Dim tasks = From ti In s_activeTasks _
                    Where ti.Value.AssignedTo = assignedTo _
                    Select ti.Value

        Console.WriteLine("Get the following tasks:")
        For Each t In tasks
            Console.WriteLine(t.TaskNumber)
        Next

        Return tasks.ToList()
    End Function

    Public Function GetTask(ByVal taskNumber As Integer) _
        As Task Implements ITaskManagerService.GetTask
        If Not s_activeTasks.ContainsKey(taskNumber) Then
            Dim msg As String = String.Format( _
                    "No task with number {0}", taskNumber)
            Console.WriteLine(msg)

            Dim fi As FaultInfo = New FaultInfo()
            fi.Reason = msg
            Dim fe As FaultException(Of FaultInfo)
            fe = New FaultException(Of FaultInfo)(fi)
            Throw fe
        End If

        Dim task As Task
        task = s_activeTasks.Item(taskNumber)
        Console.WriteLine("Getting task: {0}", _
                          task.ToString())

        Return task
    End Function

    Public Sub MarkTaskCompleted(ByVal taskNumber As Integer) _
        Implements ITaskManagerService.MarkTaskCompleted

        If Not s_activeTasks.ContainsKey(taskNumber) Then
            Dim msg As String = String.Format( _
                    "No task with number {0}", taskNumber)
            Console.WriteLine(msg)

            Dim fi As FaultInfo = New FaultInfo()
            fi.Reason = msg
            Dim fe As FaultException(Of FaultInfo)
            fe = New FaultException(Of FaultInfo)(fi)
            Throw fe
        End If

        Dim task As Task = s_activeTasks.Item(taskNumber)
        If DateTime.Now > task.DueDate Then
            task.TaskState = TaskStates.CompletedPastDueDate
        Else
            task.TaskState = TaskStates.CompletedByDueDate
        End If

        Console.WriteLine( _
                "Marked task number {0} as completed", _
                    taskNumber)

    End Sub

    ' Etc...

    Public Sub DeleteTask(ByVal taskNumber As Integer) _
        Implements ITaskManagerService.DeleteTask

        If Not s_activeTasks.ContainsKey(taskNumber) Then
            Dim msg As String = String.Format( _
                        "No task with number {0}", taskNumber)
            Console.WriteLine(msg)

            Dim fi As FaultInfo = New FaultInfo()
            fi.Reason = msg
            Dim fe As FaultException(Of FaultInfo)
            fe = New FaultException(Of FaultInfo)(fi)
            Throw fe
        End If

        s_activeTasks.Remove(taskNumber)
        Console.WriteLine( _
            "Task number {0} as deleted", taskNumber)

    End Sub

    Private Shared s_activeTasks As Dictionary(Of Integer, Task) _
                        = New Dictionary(Of Integer, Task)

    Private Shared s_nextTaskNumber As Integer = 1
End Class
