﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;

using Tasks.Entities;

namespace Tasks.Services
{
	public class TaskManagerService : ITaskManagerService
	{
        public TaskAcknowledgement AddTask(Task task)
		{
            int taskNum = s_nextTaskNumber++;

            TaskAcknowledgement ack = new TaskAcknowledgement();
            ack.TaskNumber = taskNum;

            task.TaskNumber = taskNum;

            if (task.DueDate > DateTime.Now)
                task.TaskState = TaskStates.Active;
            else
                task.TaskState = TaskStates.Overdue;

            ack.CurrentState = task.TaskState;
            if (ack.CurrentState == TaskStates.Overdue)
                ack.Comments = "Warning: this task is already overdue!";

			Console.WriteLine("Adding new task:\n{0}",task.ToString());
			s_activeTasks.Add(taskNum, task);
			return ack;
		}

        public List<Task> GetTasksByAssignedName(string assignedTo)
		{
			var tasks = from ti in s_activeTasks
						where ti.Value.AssignedTo == assignedTo
						select ti.Value;

			Console.WriteLine("Getting the following tasks:");
			foreach (var t in tasks)
				Console.WriteLine(t.TaskNumber);

			return tasks.ToList();
		}

        public Task GetTask(int taskNumber)
		{
			if (!s_activeTasks.ContainsKey(taskNumber))
			{
				string msg = string.Format("No task with number {0}", taskNumber);
				Console.WriteLine(msg);

				FaultInfo fi = new FaultInfo();
				fi.Reason = msg;
				throw new FaultException<FaultInfo>(fi);
			}

			Task task = s_activeTasks[taskNumber];

			Console.WriteLine("Getting task: {0}", task.ToString());

			return task;
		}

		public void MarkTaskCompleted(int taskNumber)
		{
			if (!s_activeTasks.ContainsKey(taskNumber))
			{
				string msg = string.Format("No task with number {0}", taskNumber);
				Console.WriteLine(msg);

				FaultInfo fi = new FaultInfo();
				fi.Reason = msg;
				throw new FaultException<FaultInfo>(fi);
			}

            Task task = s_activeTasks[taskNumber];
            if (DateTime.Now > task.DueDate)
                task.TaskState = TaskStates.CompletedPastDueDate;
            else
                task.TaskState = TaskStates.CompletedByDueDate;

			Console.WriteLine("Marked task number {0} as completed", task.TaskNumber);
		}

		public void DeleteTask(int taskNumber)
		{
			if (!s_activeTasks.ContainsKey(taskNumber))
			{
				string msg = string.Format("No task with number {0}", taskNumber);
				Console.WriteLine(msg);

				FaultInfo fi = new FaultInfo();
				fi.Reason = msg;
				throw new FaultException<FaultInfo>(fi);
			}

			s_activeTasks.Remove(taskNumber);

			Console.WriteLine("Task number {0} was deleted", taskNumber);
		}

		private static Dictionary<int, Task> s_activeTasks = new Dictionary<int, Task>();
		private static int s_nextTaskNumber = 1;
	}
}
