﻿using System;
using System.Runtime.Serialization;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Tasks.Entities
{
    [DataContract(Namespace = "http://schemas.fabrikam.com/2008/04/tasks/")]
    public enum TaskStates : int
    {
        [EnumMember()]
        Active = 0,

        [EnumMember()]
        CompletedByDueDate = 1,

        [EnumMember()]
        CompletedPastDueDate = 2,

        [EnumMember()]
        Overdue = 3,
    }

    [DataContract(Namespace = "http://schemas.fabrikam.com/tasks/")]
    public class FaultInfo
    {
        [DataMember(IsRequired = true, Order = 0)]
        public string Reason;
    }

    [DataContract(Namespace = "http://schemas.fabrikam.com/2008/04/tasks/")]
    public class Task : IExtensibleDataObject
    {
        [DataMember(IsRequired = false, Order = 0)]
        public int TaskNumber;

        [DataMember(IsRequired = true, Order = 1)]
        public string Description;

        [DataMember(IsRequired = false, Order = 2)]
        public TaskStates TaskState;

        [DataMember(IsRequired = false, Order = 3)]
        public string AssignedTo;

        [DataMember(IsRequired = false, Order = 4)]
        public string CreatedBy;

        [DataMember(IsRequired = false, Order = 5)]
        public DateTime DateCreated;

        [DataMember(IsRequired = false, Order = 6)]
        public DateTime DateLastModified;

        [DataMember(IsRequired = false, Order = 7)]
        public DateTime DueDate;

        public override string ToString()
        {
            return string.Format(
                    "Task: {0}\tAssigned to: {1}\tStatus: {2}\nDescription:\n{3}\nCreated By: {4}\tDate Created: {5}\tDate Last Modified: {6}\nDue Date: {7}",
                    TaskNumber, AssignedTo,
                    TaskState, Description,
                    CreatedBy, DateCreated, 
                    DateLastModified, DueDate);
        }

        public ExtensionDataObject ExtensionData
        {
            get
            {
                return _extensionData;
            }

            set
            {
                _extensionData = value;
            }
        }

        private ExtensionDataObject _extensionData;
    }

    [DataContract(Namespace = "http://schemas.fabrikam.com/2008/04/tasks/")]
    public class TaskAcknowledgement : IExtensibleDataObject
    {
        [DataMember(IsRequired = false, Order = 0)]
        public int TaskNumber;

        [DataMember(IsRequired = false, Order = 1)]
        public TaskStates CurrentState;

        [DataMember(IsRequired = false, Order = 2)]
        public string Comments;

        public ExtensionDataObject ExtensionData
        {
            get
            {
                return _extensionData;
            }

            set
            {
                _extensionData = value;
            }
        }

        private ExtensionDataObject _extensionData;
    }
}
