﻿using System;
using System.Xml;
using System.Threading;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace HostConsoleApp
{
    class GreetingServiceProxy : DuplexClientBase<IGreetingService>, 
                                    IGreetingService
    {
        public GreetingServiceProxy(InstanceContext inputInstance)
            : base(inputInstance, new NetTcpBinding(),
                new EndpointAddress("net.tcp://localhost:6789/service"))
        {
        }

        public void RequestGreeting(string name)
        {
            this.Channel.RequestGreeting(name);
        }
    }

    class GreetingServiceAgent : IGreetingService, 
                                IGreetingHandler, IDisposable
    {
        public GreetingServiceAgent()
        {
            try
            {
                // Setup instance context and pass it to proxy:
                InstanceContext context = new InstanceContext(this);
                _proxy = new GreetingServiceProxy(context);
                _proxy.Open();
            }

            catch
            {
                _proxy = null;
            }
        }

        public void Dispose()
        {
            if (_proxy != null)
                _proxy.Close();
        }

        public void RequestGreeting(string name)
        {
            if (_proxy != null)
                _proxy.RequestGreeting(name);
        }

        public void GreetingProduced(string greeting)
        {
            Console.WriteLine("Called back with greeting: {0}", greeting);
        }

        private GreetingServiceProxy _proxy;
    }

    class Program
    {
        static void RunService()
        {
            ServiceHost host = new ServiceHost(typeof(GreetingService));
            host.AddServiceEndpoint(typeof(IGreetingService), 
                new NetTcpBinding(), "net.tcp://localhost:6789/service");
            host.Open();
            autoEvent.Set();

            Console.WriteLine("Press any key to exit");
            Console.ReadLine();
        }

        static void Main(string[] args)
        {
            new Thread(new ThreadStart(RunService)).Start();
            autoEvent.WaitOne();

            try
            {
                GreetingServiceAgent agent = new GreetingServiceAgent();
                agent.RequestGreeting("Sally");
            }

            catch (Exception ex)
            {
                Console.WriteLine("Error: {0}", ex.Message);
            }
        }

        static AutoResetEvent autoEvent = new AutoResetEvent(false);
    }
}
