﻿Imports System.Threading
Imports System.ServiceModel
Imports System.ServiceModel.Channels

Public Class GreetingServiceProxy
    Inherits DuplexClientBase(Of IGreetingService)
    Implements IGreetingService

    Public Sub New(ByVal inputInstance As InstanceContext)
        MyBase.New(inputInstance, New NetTcpBinding(), _
            New EndpointAddress("net.tcp://localhost:6789/service"))
    End Sub

    Public Sub RequestGreeting(ByVal name As String) _
            Implements IGreetingService.RequestGreeting
        Me.Channel.RequestGreeting(name)
    End Sub
End Class

Public Class GreetingServiceAgent
    Implements IGreetingService, IGreetingHandler, IDisposable

    Public Sub New()
        Try
            ' Setup instance context and pass it to proxy:
            Dim context As New InstanceContext(Me)
            _proxy = New GreetingServiceProxy(context)
            _proxy.Open()
        Catch ex As Exception
            _proxy = Nothing
        End Try
    End Sub

    Public Sub RequestGreeting(ByVal name As String) _
            Implements IGreetingService.RequestGreeting
        If Not _proxy Is Nothing Then
            _proxy.RequestGreeting(name)
        End If
    End Sub

    Public Sub GreetingProduced(ByVal greeting As String) _
            Implements IGreetingHandler.GreetingProduced
        Console.WriteLine( _
            "Called back with greeting: {0}", greeting)
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        If Not _proxy Is Nothing Then
            _proxy.Close()
        End If
    End Sub

    Private _proxy As GreetingServiceProxy
End Class

Public Class Program

    Public Shared Sub Main()
        Dim t As Thread
        t = New Thread(New ThreadStart( _
                AddressOf Program.RunService))
        t.Start()
        autoEvent.WaitOne()

        Try
            Dim agent As New GreetingServiceAgent
            agent.RequestGreeting("Sally")
        Catch ex As Exception
            Console.WriteLine("Error: {0}", ex.Message)
        End Try

    End Sub

    Public Shared Sub RunService()
        Dim host As ServiceHost
        host = New ServiceHost(GetType(GreetingService))

        host.AddServiceEndpoint(GetType(IGreetingService), _
                New NetTcpBinding(), "net.tcp://localhost:6789/service")

        host.Open()
        autoEvent.Set()

        Console.WriteLine("Press any key to exit")
        Console.ReadLine()
    End Sub

    Public Shared autoEvent As AutoResetEvent = New AutoResetEvent(False)
End Class
