﻿using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Microsoft.MapPoint.Proxy;
using Microsoft.MapPoint.ServiceAgent;

namespace MapPointTestClient
{
    public partial class MainForm : Form
    {
        private const string DefaultDataSourceName = "MapPoint.NA";

        private MapPointServiceAgent _svcAgent = new MapPointServiceAgent();

        private Dictionary<Address, Bitmap> _cachedMaps = new Dictionary<Address, Bitmap>();

        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            InitAddressInputFields();

            _countriesComboBox.SelectedIndex = 0;
            _recentlyViewedComboBox.ComboBox.DisplayMember = "AddressLine";

            _recentlyViewedComboBox.SelectedIndexChanged += new EventHandler(_recentlyViewedComboBox_SelectedIndexChanged);
        }

        private void _recentlyViewedComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            Address addr = _recentlyViewedComboBox.SelectedItem as Address;
            if (addr != null)
            {
                Bitmap bmap = _cachedMaps[addr];
                if (bmap != null)
                    _mainPicBox.Image = bmap;
            }
        }

        private void InitAddressInputFields()
        {
            _tboxAddressLine.Text = string.Empty;
            _tboxCity.Text = string.Empty;
            _tboxProvinceOrState.Text = string.Empty;
            _tboxZipOrPostalCode.Text = string.Empty;
        }

        private void _btnViewMap_Click(object sender, EventArgs e)
        {
            Address address = new Address();

            address.AddressLine = _tboxAddressLine.Text;
            address.PrimaryCity = _tboxCity.Text;
            address.Subdivision = _tboxProvinceOrState.Text;
            address.PostalCode = _tboxZipOrPostalCode.Text;
            address.CountryRegion = _countriesComboBox.SelectedItem as string;

            MapImage img = _svcAgent.GetSizedMapByAddress(address,
                    _mainPicBox.Height, _mainPicBox.Width,DefaultDataSourceName);

            if (img != null)
            {
                // Display the map:
                Bitmap bmapImg = new Bitmap(new MemoryStream(img.MimeData.Bits));
                _mainPicBox.Image = bmapImg;

                // Cache the image:
                _cachedMaps.Add(address, bmapImg);

                // And update combo box:
                _recentlyViewedComboBox.Items.Add(address);
                _recentlyViewedComboBox.SelectedIndex = _recentlyViewedComboBox.Items.Count - 1;
            }
            else
            {
                MessageBox.Show(
                    "No map found for address:\n" +
                            MapPointServiceAgent.AddressToString(address),
                     "Error");
            }

            InitAddressInputFields();
        }
    }
}
