﻿using System;
using System.Security.Principal;
using System.Runtime.Remoting.Messaging;
using System.Configuration;
using System.Net;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Microsoft.MapPoint.Proxy;

namespace Microsoft.MapPoint.ServiceAgent
{
    public class MapPointServiceAgent
    {
        public Location GetLocationByAddress(Address address, string dataSourceName)
        {
            FindServiceSoapClient findSvcProxy = InitFindServiceProxy();
            FindAddressSpecification addrSpec = new FindAddressSpecification();

            addrSpec.InputAddress = address;
            addrSpec.DataSourceName = dataSourceName;

            // Note: CustomerInfoFindHeader & UserInfoFindHeader can be null
            // here since we are happy with defaults
            FindResults results;
            try
            {
                results = findSvcProxy.FindAddress(null, null, addrSpec);
            }
            catch
            {
                results = null;
            }

            Location res = null;
            if (results != null && results.NumberFound > 0 && 
                        results.Results[0].FoundLocation != null)
            {
                res = results.Results[0].FoundLocation;
            }

            return res;
        }

        public MapImage GetMapByLocation(Location location)
        {
            if (location == null)
                return null;

            RenderServiceSoapClient renderSvcProxy = InitRenderServiceProxy();

            MapSpecification mapSpec = new MapSpecification();
            mapSpec.DataSourceName = location.DataSourceName;
            mapSpec.Views = new MapView[] { location.BestMapView.ByBoundingRectangle };

            MapImage[] mapImages;
            try
            {
                mapImages = renderSvcProxy.GetMap(null, null, mapSpec);
            }
            catch
            {
                mapImages = null;
            }

            MapImage res = null;
            if (mapImages != null && mapImages.Length > 0)
                res = mapImages[0];

            return res;
        }

        public MapImage GetMapByLocation(Location location, double mapHeight, double mapWidth)
        {
            if (location == null)
                return null;

            RenderServiceSoapClient renderSvcProxy = InitRenderServiceProxy();

            MapSpecification mapSpec = new MapSpecification();
            mapSpec.DataSourceName = location.DataSourceName;

            // Init view:
            ViewByHeightWidth vbh = new ViewByHeightWidth();
            vbh.Height = mapHeight;
            vbh.Width = mapWidth;
            vbh.CenterPoint = location.LatLong;

            mapSpec.Views = new MapView[] { vbh };

            // Init options:
            mapSpec.Options = new MapOptions();

            mapSpec.Options.Format = new ImageFormat();
            mapSpec.Options.Format.Height = (int) mapHeight;
            mapSpec.Options.Format.Width = (int) mapWidth;

            mapSpec.Options.Zoom = 0.001;

            // Init pushpin:
            Pushpin pin = new Pushpin();
            pin.IconDataSource = "MapPoint.Icons";
            pin.IconName = "1";
            pin.Label = location.Address.AddressLine;
            pin.LatLong = location.LatLong;
            mapSpec.Pushpins = new Pushpin[] { pin };

            MapImage[] mapImages;
            try
            {
                mapImages = renderSvcProxy.GetMap(null, null, mapSpec);
            }
            catch
            {
                mapImages = null;
            }

            MapImage res = null;
            if (mapImages != null && mapImages.Length > 0)
                res = mapImages[0];

            return res;
        }

        public MapImage GetSizedMapByAddress(Address address, 
                    double mapHeight, double mapWidth, string dataSourceName)
        {
            Location location = GetLocationByAddress(address, dataSourceName);
            return GetMapByLocation(location,mapHeight,mapWidth);
        }

        public MapImage GetMapByAddress(Address address, string dataSourceName)
        {
            Location location = GetLocationByAddress(address, dataSourceName);
            return GetMapByLocation(location);
        }

        public static string AddressToString(Address address)
        {
            return string.Format("{0}, {1}, {2}, {3}, {4}", address.AddressLine, 
                                    address.PrimaryCity, 
                                    address.Subdivision,
                                    address.PostalCode, 
                                    address.CountryRegion);
        }

        private FindServiceSoapClient InitFindServiceProxy()
        {
            FindServiceSoapClient findSvcProxy = new FindServiceSoapClient();

            findSvcProxy.ClientCredentials.HttpDigest.ClientCredential = new NetworkCredential(
                ConfigurationManager.AppSettings[MapPointWebServiceIDKey],
                ConfigurationManager.AppSettings[MapPointWebServicePasswordKey]);

            findSvcProxy.ClientCredentials.HttpDigest.AllowedImpersonationLevel = TokenImpersonationLevel.Impersonation;

            return findSvcProxy;
        }

        private RenderServiceSoapClient InitRenderServiceProxy()
        {
            RenderServiceSoapClient renderSvcProxy = new RenderServiceSoapClient();

            renderSvcProxy.ClientCredentials.HttpDigest.ClientCredential = new NetworkCredential(
                ConfigurationManager.AppSettings[MapPointWebServiceIDKey],
                ConfigurationManager.AppSettings[MapPointWebServicePasswordKey]);

            renderSvcProxy.ClientCredentials.HttpDigest.AllowedImpersonationLevel = TokenImpersonationLevel.Impersonation;

            return renderSvcProxy;
        }

        private const string MapPointWebServiceIDKey = "MapPointWebServiceID";
        private const string MapPointWebServicePasswordKey = "MapPointWebServicePassword";
    }
}
