﻿Imports System.Collections.Generic
Imports System.IO

Imports Microsoft.MapPoint.ServiceAgent
Imports Microsoft.MapPoint.ServiceAgent.Proxy


Public Class MainForm
    Private Const DefaultDataSourceName As String = "MapPoint.NA"
    Private _svcAgent As MapPointServiceAgent = New MapPointServiceAgent()
    Private _cachedMaps As Dictionary(Of Address, Bitmap) = _
                                    New Dictionary(Of Address, Bitmap)()

    Private Sub InitAddressInputFields()
        _tboxAddressLine.Text = String.Empty
        _tboxCity.Text = String.Empty
        _tboxProvinceOrState.Text = String.Empty
        _tboxZipOrPostalCode.Text = String.Empty
    End Sub


    Private Sub _btnViewMap_Click(ByVal sender As System.Object, _
            ByVal e As System.EventArgs) Handles _btnViewMap.Click
        Dim address As Address = New Address()

        address.AddressLine = _tboxAddressLine.Text
        address.PrimaryCity = _tboxCity.Text
        address.Subdivision = _tboxProvinceOrState.Text
        address.PostalCode = _tboxZipOrPostalCode.Text
        address.CountryRegion = CType( _
                    _countriesComboBox.SelectedItem, String)

        _svcAgent.BeginGetSizedMapByAddress(address, _
                _mainPicBox.Height, _mainPicBox.Width, _
                DefaultDataSourceName, _
                AddressOf Me.HandleMapImageAvailableForDisplay)

        InitAddressInputFields()

        _mainStatusLabel.Text = String.Format( _
                "Retrieving map for address {0}", _
                MapPointServiceAgent.AddressToString(address))
    End Sub

    Private Sub HandleMapImageAvailableForDisplay( _
                ByVal img As MapImage, ByVal address As Address)
        If Not img Is Nothing Then
            _mainStatusLabel.Text = String.Format( _
                    "Map for address {0} was retrieved", _
                    MapPointServiceAgent.AddressToString(address))

            ' Display the map:
            Dim bmapImg As Bitmap
            bmapImg = New Bitmap( _
                    New MemoryStream(img.MimeData.Bits))
            _mainPicBox.Image = bmapImg

            ' Cache the image:
            _cachedMaps.Add(address, bmapImg)

            ' And update combo box:
            _recentlyViewedComboBox.Items.Add(address)
            _recentlyViewedComboBox.SelectedIndex = _
                _recentlyViewedComboBox.Items.Count - 1
        Else
            _mainStatusLabel.Text = String.Format( _
                "There was an error retrieving map for address {0}", _
                MapPointServiceAgent.AddressToString(address))

            MessageBox.Show("No map found for address:\n" + _
                    MapPointServiceAgent.AddressToString(address), _
                    "Error")
        End If
    End Sub

    Private Sub MainForm_Load(ByVal sender As System.Object, _
                ByVal e As System.EventArgs) Handles MyBase.Load
        InitAddressInputFields()

        _countriesComboBox.SelectedIndex = 0
        _recentlyViewedComboBox.ComboBox.DisplayMember = "AddressLine"

        AddHandler _recentlyViewedComboBox.SelectedIndexChanged, _
                    AddressOf _recentlyViewedComboBox_SelectedIndexChanged
    End Sub

    Private Sub _recentlyViewedComboBox_SelectedIndexChanged( _
                        ByVal sender As Object, ByVal e As EventArgs)
        Dim addr As Address
        addr = CType(_recentlyViewedComboBox.SelectedItem, Address)

        If Not addr Is Nothing Then
            Dim bmap As Bitmap
            bmap = _cachedMaps(addr)
            If Not bmap Is Nothing Then
                _mainPicBox.Image = bmap
            End If
        End If
    End Sub
End Class
