﻿Imports System.ServiceModel
Imports System.Configuration
Imports System.Security.Principal
Imports System.Net

Imports Microsoft.MapPoint.ServiceAgent.Proxy

Public Class MapPointServiceAgent

    Public Function GetLocationByAddress( _
            ByVal address As Address, ByVal dataSourceName As String) _
                As Location
        Dim findSvcProxy As FindServiceSoapClient
        findSvcProxy = InitFindServiceProxy()

        Dim addrSpec As FindAddressSpecification
        addrSpec = New FindAddressSpecification()
        addrSpec.InputAddress = address
        addrSpec.DataSourceName = dataSourceName

        ' Note: CustomerInfoFindHeader & UserInfoFindHeader
        ' can be null here since we are happy with defaults
        Dim results As FindResults = Nothing
        Try
            results = findSvcProxy.FindAddress(Nothing, Nothing, addrSpec)
        Catch
            results = Nothing
        End Try

        Dim res As Location = Nothing
        If Not results Is Nothing Then
            If results.NumberFound > 0 Then
                If Not results.Results(0).FoundLocation Is Nothing Then
                    res = results.Results(0).FoundLocation
                End If
            End If
        End If

        Return res
    End Function

    Public Function GetMapByLocation( _
                ByVal location As Location) As MapImage

        If location Is Nothing Then
            Return Nothing
        End If

        Dim renderSvcProxy As RenderServiceSoapClient
        renderSvcProxy = InitRenderServiceProxy()

        Dim mapSpec As New MapSpecification()
        mapSpec.DataSourceName = location.DataSourceName

        Dim views(0) As MapView
        views(0) = location.BestMapView.ByBoundingRectangle
        mapSpec.Views = views

        Dim mapImages() As MapImage
        Try
            mapImages = renderSvcProxy.GetMap(Nothing, Nothing, mapSpec)
        Catch
            mapImages = Nothing
        End Try

        Dim res As MapImage = Nothing
        If Not mapImages Is Nothing Then
            If mapImages.Length > 0 Then
                res = mapImages(0)
            End If
        End If

        Return res
    End Function


    Public Function GetMapByLocation( _
            ByVal location As Location, ByVal mapHeight As Double, _
            ByVal mapWidth As Double) As MapImage

        If location Is Nothing Then
            Return Nothing
        End If

        Dim renderSvcProxy As RenderServiceSoapClient
        renderSvcProxy = InitRenderServiceProxy()

        Dim mapSpec As New MapSpecification
        mapSpec.DataSourceName = location.DataSourceName

        ' Init view:
        Dim vbh As New ViewByHeightWidth()
        vbh.Height = mapHeight
        vbh.Width = mapWidth
        vbh.CenterPoint = location.LatLong

        Dim views(0) As MapView
        views(0) = vbh
        mapSpec.Views = views

        ' Init options:
        mapSpec.Options = New MapOptions()
        mapSpec.Options.Format = New ImageFormat()
        mapSpec.Options.Format.Height = Convert.ToInt32(mapHeight)
        mapSpec.Options.Format.Width = Convert.ToInt32(mapWidth)

        mapSpec.Options.Zoom = 0.001

        ' Init pushpin:
        Dim pin As New Pushpin()
        pin.IconDataSource = "MapPoint.Icons"
        pin.IconName = "1"
        pin.Label = location.Address.AddressLine
        pin.LatLong = location.LatLong

        Dim pins(0) As Pushpin
        pins(0) = pin
        mapSpec.Pushpins = pins

        Dim mapImages() As MapImage
        Try
            mapImages = renderSvcProxy.GetMap(Nothing, Nothing, mapSpec)
        Catch ex As Exception
            mapImages = Nothing
        End Try

        Dim res As MapImage = Nothing
        If Not mapImages Is Nothing Then
            If mapImages.Length > 0 Then
                res = mapImages(0)
            End If
        End If

        Return res
    End Function


    Public Function GetSizedMapByAddress( _
            ByVal address As Address, ByVal mapHeight As Double, _
            ByVal mapWidth As Double, ByVal dataSourceName As String) _
                As MapImage
        Dim location As Location
        location = GetLocationByAddress(address, dataSourceName)
        Return GetMapByLocation(location, mapHeight, mapWidth)
    End Function

    Public Function GetMapByAddress( _
        ByVal address As Address, ByVal dataSourceName As String) _
            As MapImage
        Dim location As Location
        location = GetLocationByAddress(address, dataSourceName)
        Return GetMapByLocation(location)
    End Function

    Public Shared Function AddressToString( _
                    ByVal address As Address) As String
        Return String.Format("{0}, {1}, {2}, {3}, {4}", _
                            address.AddressLine, _
                            address.PrimaryCity, _
                            address.Subdivision, _
                            address.PostalCode, _
                            address.CountryRegion)
    End Function

    Private Function InitFindServiceProxy() As FindServiceSoapClient
        Dim findSvcProxy As FindServiceSoapClient
        findSvcProxy = New FindServiceSoapClient()

        findSvcProxy.ClientCredentials.HttpDigest.ClientCredential = _
                New NetworkCredential( _
                    ConfigurationManager.AppSettings(MapPointWebServiceIDKey), _
                    ConfigurationManager.AppSettings(MapPointWebServicePasswordKey))

        findSvcProxy.ClientCredentials.HttpDigest.AllowedImpersonationLevel = _
                            TokenImpersonationLevel.Impersonation

        Return findSvcProxy
    End Function

    Private Function InitRenderServiceProxy() As RenderServiceSoapClient
        Dim renderSvcProxy As RenderServiceSoapClient
        renderSvcProxy = New RenderServiceSoapClient()

        renderSvcProxy.ClientCredentials.HttpDigest.ClientCredential = _
                New NetworkCredential( _
                    ConfigurationManager.AppSettings(MapPointWebServiceIDKey), _
                    ConfigurationManager.AppSettings(MapPointWebServicePasswordKey))

        renderSvcProxy.ClientCredentials.HttpDigest.AllowedImpersonationLevel = _
                            TokenImpersonationLevel.Impersonation

        Return renderSvcProxy
    End Function

    Private Const MapPointWebServiceIDKey As String = "MapPointWebServiceID"
    Private Const MapPointWebServicePasswordKey As String = "MapPointWebServicePassword"
End Class

