﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IdentityModel.Policy;
using System.IdentityModel.Claims;
using System.Diagnostics;
using System.Security.Principal;

namespace DemoService
{
    public class SpecificRoleAuthorizationPolicy : IAuthorizationPolicy
    {
        private string id;
        private ClaimSet issuer;

        public SpecificRoleAuthorizationPolicy()
        {
            id = Guid.NewGuid().ToString();
            issuer = ClaimSet.System;
        }

        public string Id
        {
            get { return id; }
        }

        public ClaimSet Issuer
        {
            get { return issuer; }
        }

        public bool Evaluate(EvaluationContext context, ref object state)
        {
            object property;
            if (!context.Properties.TryGetValue("Identities", out property)) 
                return false;

            List<IIdentity> identities = property as List<IIdentity>;
            if (identities == null || identities.Count == 0)
                return false;
            
            context.Properties["Principal"] = new GenericPrincipal(identities[0], new string[] { "HeaderPeople" });

            return true;

        }
    }

}
